package it.uniroma1.lcl.wosit;


import it.uniroma1.lcl.evaluation.src.core.WSIEvaluator;
import it.uniroma1.lcl.jlt.DBConfiguration;
import it.uniroma1.lcl.jlt.collocs.Lexicon;
import it.uniroma1.lcl.jlt.dev.util.condition.PercentageCondition;
import it.uniroma1.lcl.jlt.util.Strings;
import it.uniroma1.lcl.wosit.data.sorters.CardinalitySorter;
import it.uniroma1.lcl.wosit.data.sorters.Sorter;
import it.uniroma1.lcl.wosit.dataset.Dataset;
import it.uniroma1.lcl.wosit.metrics.Metric;
import it.uniroma1.lcl.wosit.metrics.TokenOverlap;
import it.uniroma1.lcl.wosit.metrics.TypeMetrics;
import it.uniroma1.lcl.wosit.snippet.SnippetAssociator;
import it.uniroma1.lcl.wosit.snippet.SnippetClustering;
import it.uniroma1.lcl.wosit.wsi.algorithms.ChineseWhispers;
import it.uniroma1.lcl.wosit.wsi.algorithms.ClusteringAlgorithm;
import it.uniroma1.lcl.wosit.wsi.algorithms.HyperLex;
import it.uniroma1.lcl.wosit.wsi.data.Cluster;
import it.uniroma1.lcl.wosit.wsi.data.Clustering;
import it.uniroma1.lcl.wosit.wsi.data.UndirectedWeightedGraph;
import it.uniroma1.lcl.wosit.wsi.data.WordGraph;

import java.lang.reflect.InvocationTargetException;

import weka.associations.FPGrowth.AssociationRule.METRIC_TYPE;

public class WoSITdemo {

	static private WoSITdemo instance;

	/**
	 * Used to access {@link WoSITdemo}
	 * 
	 * @return an instance of {@link WoSITdemo}
	 */
	public static synchronized WoSITdemo getInstance()
	{
		if (instance == null) instance = new WoSITdemo();
		return instance;
	}

	public static <E> Clustering<E> applyAlgorithm(ClusteringAlgorithm<E> algorithm)
	{
		System.out.println(Strings.decorate(algorithm.getClass().getSimpleName() + " algorithm"));
		
		algorithm.makeClustering();
		
		Clustering<E> clustering = algorithm.getClustering();
		
		for(Cluster<E> cluster : clustering.getWordClusters())
			System.out.println("==============\n" + cluster.getWords());
		
		return clustering;
	}

//	public static void test_creaGraphWithConfigFile(String targetWord)
//	{
//		//create graph reading the file config
//		WordGraph gh = new WordGraph();
//		gh = WordGraph.createWordGraph(Dataset.getInstance().getTopicDataset(targetWord).getAllSnippetsWords());
//		gh.setTargetWord(targetWord);
//		
//		gh.saveToDOT("dot/test_graph");
//		gh.saveToSER(targetWord);
//	}

//	public static void createGraph()
//	{
//		String line= null;
//
//		WordGraph gh = new WordGraph();
//		IntegerCounter<String> listV = new  IntegerCounter<String>(); 
//		try
//		{
//			BufferedReader reader = new BufferedReader(new FileReader("resources/grafo.txt"));
//
//			int conta = 0;
//			while((line = reader.readLine())!= null)
//			{
//				conta++;
//
//				String e1 = line.split("\t")[0];
//				String e2 = line.split("\t")[1];
//				String w = line.split("\t")[2];
//				if(Double.parseDouble(w)<0.1) continue;
//
//				//Write.writeOutputWithAppend("Analisi.txt", txt);
//				if(w.equals("NaN"))continue;
//
//				listV.count(e1);
//				listV.count(e2);
//
//				//if(Double.parseDouble(w)<0.03)continue;
//				if(!gh.containsVertex(e1))gh.addVertex(e1);
//				if(!gh.containsVertex(e2))gh.addVertex(e2);
//				/// System.out.println(e1);
//				if(conta%10000 == 0)System.out.println("numero controlli: "+conta);
//				if(!e2.equals(e1))
//					gh.addEdge(e1, e2,Double.parseDouble(w));
//			}
//		}
//		catch (FileNotFoundException e1) {e1.printStackTrace();}
//		catch (IOException e){e.printStackTrace();}
//
//		//		gh.setTargetWord("Apple");
//		//	 	gh.removeNode(gh.getTargetWord());
//		//		ListVertexCooccurrence listVertFreq = new ListVertexCooccurrence ();
//		//		for(String nodes: listV.keySet()){
//		//			VertexCooccurrence vctemp = new VertexCooccurrence(nodes,(long)listV.get(nodes));
//		//			listVertFreq.add(vctemp);
//		//		}
//		//		gh.setListVertexCooccurrence(listVertFreq);
//		//		System.out.println("listV "+listV.size()); 
//		// 		HyperLex mst = new HyperLex(gh);
//		//		mst.makeClustering();
//		//		for (WordCluster cluster : mst.getClustering().getWordClusters())
//		//			{
//		//				System.out.println("==============");
//		//				for (String word : cluster.getWords())
//		//					System.out.println(""+word);
//		//			}
//		
//		MST mst = new MST(gh);
//		mst.makeClustering();
//		for (WordCluster cluster : mst.getClustering().getWordClusters())
//		{
//			System.out.println("==============");
//			for (String word : cluster.getWords())
//				System.out.println(""+word);
//		}
//
//		gh.saveToDOT("dot/disambuigatore");
//		gh.saveToSER("dadda");
//		System.out.println(gh.getTargetWord());
//	} 

	public static WordGraph createGraphFromTxt(String filename)
	{
		return WordGraph.createFromTxtFile(filename);
	}
	
	public static WordGraph createGraphFromAPI()
	{
		System.out.println("Creating a graph programmatically...");
		
		WordGraph graph = new WordGraph();
		graph.addVertex("A");
		graph.addVertex("B");
		graph.addVertex("C");
		graph.addVertex("D");
		
		graph.addEdge("A", "B", 1);
		graph.addEdge("A", "C", 2);
		graph.addEdge("C", "D", 0.5);
		graph.addEdge("A", "C", 2);
		
		System.out.println("Graph programmatically created.");
		System.out.println("|V|="+graph.vertexSet().size());
		System.out.println("|E|="+graph.edgeSet().size());
		
		return graph;
	}
	
	public static WordGraph createGraphFromDatabase(String targetWord)
	{
		DBConfiguration configDB = DBConfiguration.getInstance();

		String databaseID = "WEBCO";
		String location = "localhost";
		String port = "3306";
		String user = "root";
		String password = "maudmysql";
		
		configDB.setDatabases(databaseID, location, port, user, password);
		
		return WordGraph.createWordGraph(targetWord, Dataset.getInstance(), configDB);
	}
	
	public static WordGraph createGraphFromSerializedFile(String ID)
	{
		return WordGraph.loadFromSer(ID);
	}
	
	public static void testCompleteDataset(Dataset dataset, Class<? extends ClusteringAlgorithm> algorithmClass, Metric metric, Sorter sorter) throws SecurityException, NoSuchMethodException, IllegalArgumentException, InstantiationException, IllegalAccessException, InvocationTargetException
	{
		System.out.println("COMPLETE TEST WITH DATASET: " + dataset.getDatasetName());
		System.out.println(dataset.getQueries().size() + " TOTAL QUERIES.");
		
		PercentageCondition condition = new PercentageCondition(1, dataset.getQueries().size());
		condition.setId(dataset.getDatasetName() + " COMPLETION");
		
		for(String query : dataset.getQueries())
		{
			condition.triggerAndIncrement();
			
			System.out.println(Strings.decorate("QUERY: " + query));
			
			WordGraph graph = createGraphFromDatabase(query);
			ClusteringAlgorithm algorithm = algorithmClass.getConstructor(new Class[]{UndirectedWeightedGraph.class}).newInstance(graph);
			algorithm.makeClustering();
			Clustering clustering = algorithm.getClustering();
			
			SnippetClustering snippetClusterAssociation = SnippetAssociator.getInstance().associateSnippet(dataset.getTopicDataset(graph.getTargetWord()), clustering, metric);
			snippetClusterAssociation.exportClusters("output/complete.test.txt", true);
		}
		
		WSIEvaluator.getInstance(dataset.getDatasetName(), "output/complete.test.txt");
	}

	public static void main(String[] args) throws SecurityException, IllegalArgumentException, NoSuchMethodException, InstantiationException, IllegalAccessException, InvocationTargetException
	{
//		testCompleteDataset(Dataset.getInstance(), ChineseWhispers.class, new TokenOverlap(), new CardinalitySorter());
//		System.exit(0);
		
		String targetWord = "excalibur";
		
		/*
		 * Create the graph
		 * 1. From a text file;
		 * 2. Programmatically;
		 * 3. From your cooccurrence database.
		 * 4. From a serialized version
		 */
		WordGraph graph = createGraphFromTxt(targetWord + "_graph.txt");
//		WordGraph graph = createGraphFromAPI();
//		WordGraph graph = createGraphFromSerializedFile("toy");
//		WordGraph graph = createGraphFromDatabase("excalibur");
//		graph.saveToTxt(targetWord + "_graph.txt");
		
		/*
		 * Apply your favourite WSI algorithm
		 * 1. B-MST
		 * 2. HyperLex
		 * 3. Chinese Whispers
		 * 4. SquaT++
		 * 5. Dina
		 * 6. SmallRing
		 * 7. Lin98
		 */
//		applyAlgorithm(new BMST(graph));
//		applyAlgorithm(new HyperLex(graph, Lexicon.getInstance()));
//		applyAlgorithm(new ChineseWhispers(graph));
//		applyAlgorithm(new SquaT(graph));
//		applyAlgorithm(new SmallRing(graph));
//		applyAlgorithm(new Dina(graph.getTargetWord()));
//		applyAlgorithm(new Lin98(graph.getTargetWord()));
		
		/*
		 * Apply the snippet associator
		 * 1. Get the snippet-cluster association
		 * 2. Print the snippet-cluster association
		 * 3. Export the snippet-cluster association
		 */
//		Clustering<String> clustering = applyAlgorithm(new ChineseWhispers<String>(graph));
//		Clustering<String> clustering = applyAlgorithm(new BMST(graph));
		Clustering<String> clustering = applyAlgorithm(new HyperLex(graph, Lexicon.getInstance()));
		
		Dataset dataset = Dataset.getInstance();
		Metric metric = TypeMetrics.DEGREE_OVERLAP.getMetric();
		SnippetClustering snippetClusterAssociation = SnippetAssociator.getInstance().associateSnippet(dataset.getTopicDataset(graph.getTargetWord()), clustering, metric);
		System.out.println(Strings.decorate("SNIPPET CLUSTERING") + "\n" + snippetClusterAssociation.toString());
//		System.out.println(Strings.decorate("EXPORTING SNIPPET CLUSTERING"));
//		snippetClusterAssociation.exportClusters("output/testAll.txt");
//		
//		/*
//		 * Evaluate the clustering
//		 */
//		WSIEvaluator.getInstance(Configuration.getInstance().getDatasetLocation() + Configuration.getInstance().getDatasetName(), "output/testAll.txt");
	}
}
